(function() {
	var Grit, EventEmitter;

	/*
	 * require in NodeJS's native EventEmitter class,
	 * or emulate it in the browser.
	 */

	if(typeof exports !== 'undefined') {
		EventEmitter = require('events').EventEmitter;
	} else {
		EventEmitter = function() {
			var handlers = {};

			this.on = this.addListener = function(e, callback) {
				if(!handlers.hasOwnProperty(e)) {
					handlers[e] = [];
				}
				handlers[e].push(callback);
			};

			this.removeListener = function(e, callback) {
				var handleList, index, length, curr;
				if(handlers.hasOwnProperty(e)) {
					handleList = handlers[e];
					for(index = 0, length = handleList.length; index < length; index++) {
						curr = handleList[index];
						if(curr === callback) {
							handleList.splice(index, 1);
							if(handleList.length === 0)
								delete handlers[e];
							return;
						}	
					}
				}
			};

			this.emit = function(e) {
				var handleList, index, length, args;
				if(handlers.hasOwnProperty(e)) {
					handleList = handlers[e];
					args = Array.prototype.slice(this, arguments).slice(1);
					for(index = 0, length = handleList.length; index < length; index++)
						handleList[index].call(this, args);
				}
			};
		};
	}

	/*
	 * The model class itself.
	 */

	Grit = function() {
		EventEmitter.call(this);
		var data = {};
		var validations = {};

		this.get = function(name) {
			if(data.hasOwnProperty(name))
				return data[name];
			return null;
		};
		this.set = function(name, value, silent) {
			var validators, valid, index, length;
			valid = false;
			if(!validations.hasOwnProperty(name))
				valid = true;
			else {
				validators = validations[name];
				valid = true;
				for(index = 0, length = validators.length; index < length; index++) {
					valid = valid && validators[index](value);
					if(!valid)
						break;
				}
			}

			if(valid) {
				if(typeof silent === 'undefined' || !silent)
					this.emit('change:' + name, this, value);
				data[name] = value;
				this.sync(this, data, name);
				return value;
			}

			return null;
		};

		this.validate = function(name, validator) {
			if(!validations.hasOwnProperty(name))
				validations[name] = [];
			validations[name].push(validator);
		};
		this.removeValidation = function(name, validator) {
		};
	};
	Grit.prototype = new EventEmitter();
	Grit.prototype.constructor = Grit;

	// sync is left empty -- redefine it to automate
	// changes being synced to the server or db.
	Grit.prototype.sync = function(model, data, property) {
	};

	// load is left empty -- redefine it to automate
	// loading things from the server or db. load
	// is never auto-called, unlike sync()
	Grit.prototype.load = function() {
	};

	/* 
	 * Export
	 */

	if(typeof exports !== 'undefined') {
		exports = module.exports = Grit;
	} else {
		window.Grit = Grit;
	}
}());
