vows = require 'vows'
assert = require 'assert'
Grit = require '../index.js'

batch = vows.describe('Grit').addBatch
	'Grit objects':
		topic: () -> Grit
		
		'should return null for uninitialized data': (Grit) ->
			g = new Grit()
			assert.equal g.get('test'), null
		
		'should return the correct data when accessing previously set data': (Grit) ->
			g = new Grit()
			g.set 'test', 500
			assert.equal g.get('test'), 500

		'should return the value being set from a set function': (Grit) ->
			g = new Grit()
			assert.equal g.set('test', 500), 500

		'should fire change events when setting data': (Grit) ->
			g = new Grit()
			called = false
			g.on 'change:test', () ->
				called = true
			g.set 'test', 500
			assert.isTrue called

		'should not fire unrelated change events when setting data': (Grit) ->
			g = new Grit()
			called = false
			g.on 'change:nope', () ->
				called = true
			g.set 'test', 500
			assert.isFalse called

		'should pass the model into change handlers as the first argument': (Grit) ->
			g = new Grit()
			g.on 'change:test', (model) ->
				assert.equal model, g
			g.set 'test', 500

		'should pass the new value into change handlers as the second argument': (Grit) ->
			g = new Grit()
			g.on 'change:test', (model, value) ->
				assert.equal value, 500
			g.set 'test', 500

		'should not have changed the old value prior to a change callback': (Grit) ->
			g = new Grit()
			g.on 'change:test', (model) ->
				assert.isNull model.get('test')
			g.set 'test', 500

		'should change the property after the change callback': (Grit) ->
			g = new Grit()
			g.on 'change:test', () ->
			g.set 'test', 500
			assert.equal g.get('test'), 500

		'should not emit the change event if a truthy value is passed as the silent parameter to set': (Grit) ->
			g = new Grit()
			called = false
			g.on 'change:test', () ->
				called = true
			g.set 'test', 500, true
			assert.isFalse called

		'should call validation functions when associated data is changed': (Grit) ->
			g = new Grit()
			called = false
			g.validate 'test', (value) ->
				called = true
			g.set 'test', 500
			assert.isTrue called

		'should change the value being set if the validation function returns true': (Grit) ->
			g = new Grit()
			g.validate 'test', (value) -> true
			g.set 'test', 500
			assert.equal g.get('test'), 500

		'should not change the value being set if the validation function returns false': (Grit) ->
			g = new Grit()
			g.validate 'test', (value) -> false
			g.set 'test', 500
			assert.isNull g.get('test')

		'should call the sync function after setting data': (Grit) ->
			g = new Grit()
			called = false
			g.sync = () ->
				called = true
			g.set 'test', 500
			assert.isTrue called

		'should pass the model to the sync function': (Grit) ->
			g = new Grit()
			g.sync = (model) ->
				assert.equal model, g
			g.set 'test', 500

		'should pass all of the model\'s data to the sync function': (Grit) ->
			g = new Grit()
			g.set 'test', 500
			g.sync = (model, data) ->
				assert.equal data.test, 500
			g.set 'beam', 200

		'should pass the current property name being changed to the sync function': (Grit) ->
			g = new Grit()
			g.sync = (model, data, property) ->
				assert.equal property, 'test'
			g.set 'test', 500

batch.run()
