var Grit, assert, batch, vows;
vows = require('vows');
assert = require('assert');
Grit = require('../index.js');
batch = vows.describe('Grit').addBatch({
  'Grit objects': {
    topic: function() {
      return Grit;
    },
    'should return null for uninitialized data': function(Grit) {
      var g;
      g = new Grit();
      return assert.equal(g.get('test'), null);
    },
    'should return the correct data when accessing previously set data': function(Grit) {
      var g;
      g = new Grit();
      g.set('test', 500);
      return assert.equal(g.get('test'), 500);
    },
    'should return the value being set from a set function': function(Grit) {
      var g;
      g = new Grit();
      return assert.equal(g.set('test', 500), 500);
    },
    'should fire change events when setting data': function(Grit) {
      var called, g;
      g = new Grit();
      called = false;
      g.on('change:test', function() {
        return called = true;
      });
      g.set('test', 500);
      return assert.isTrue(called);
    },
    'should not fire unrelated change events when setting data': function(Grit) {
      var called, g;
      g = new Grit();
      called = false;
      g.on('change:nope', function() {
        return called = true;
      });
      g.set('test', 500);
      return assert.isFalse(called);
    },
    'should pass the model into change handlers as the first argument': function(Grit) {
      var g;
      g = new Grit();
      g.on('change:test', function(model) {
        return assert.equal(model, g);
      });
      return g.set('test', 500);
    },
    'should pass the new value into change handlers as the second argument': function(Grit) {
      var g;
      g = new Grit();
      g.on('change:test', function(model, value) {
        return assert.equal(value, 500);
      });
      return g.set('test', 500);
    },
    'should not have changed the old value prior to a change callback': function(Grit) {
      var g;
      g = new Grit();
      g.on('change:test', function(model) {
        return assert.isNull(model.get('test'));
      });
      return g.set('test', 500);
    },
    'should change the property after the change callback': function(Grit) {
      var g;
      g = new Grit();
      g.on('change:test', function() {});
      g.set('test', 500);
      return assert.equal(g.get('test'), 500);
    },
    'should not emit the change event if a truthy value is passed as the silent parameter to set': function(Grit) {
      var called, g;
      g = new Grit();
      called = false;
      g.on('change:test', function() {
        return called = true;
      });
      g.set('test', 500, true);
      return assert.isFalse(called);
    },
    'should call validation functions when associated data is changed': function(Grit) {
      var called, g;
      g = new Grit();
      called = false;
      g.validate('test', function(value) {
        return called = true;
      });
      g.set('test', 500);
      return assert.isTrue(called);
    },
    'should change the value being set if the validation function returns true': function(Grit) {
      var g;
      g = new Grit();
      g.validate('test', function(value) {
        return true;
      });
      g.set('test', 500);
      return assert.equal(g.get('test'), 500);
    },
    'should not change the value being set if the validation function returns false': function(Grit) {
      var g;
      g = new Grit();
      g.validate('test', function(value) {
        return false;
      });
      g.set('test', 500);
      return assert.isNull(g.get('test'));
    },
    'should call the sync function after setting data': function(Grit) {
      var called, g;
      g = new Grit();
      called = false;
      g.sync = function() {
        return called = true;
      };
      g.set('test', 500);
      return assert.isTrue(called);
    },
    'should pass the model to the sync function': function(Grit) {
      var g;
      g = new Grit();
      g.sync = function(model) {
        return assert.equal(model, g);
      };
      return g.set('test', 500);
    },
    'should pass all of the model\'s data to the sync function': function(Grit) {
      var g;
      g = new Grit();
      g.set('test', 500);
      g.sync = function(model, data) {
        return assert.equal(data.test, 500);
      };
      return g.set('beam', 200);
    },
    'should pass the current property name being changed to the sync function': function(Grit) {
      var g;
      g = new Grit();
      g.sync = function(model, data, property) {
        return assert.equal(property, 'test');
      };
      return g.set('test', 500);
    }
  }
});
batch.run();